Module error_codes

Module error_codes 

Available on non-WebAssembly only.
Standard error codes for validation with client elicitation support. Standard error codes for validation with client elicitation support

Provides a consistent set of error codes that clients can use to automate UI elicitation and provide better user experiences.

Structs

ValidationError
Structured validation error with elicitation support

Enums

ValidationErrorCode
Standard validation error codes for MCP tools

Traits

IntoValidationError
Helper trait for creating validation errors

Functions

invalid_format
Create an invalid format error
missing_field
Create a missing field error
not_allowed
Create a not allowed error
out_of_range
Create an out of range error
pattern_mismatch
Create a pattern mismatch error
security_violation
Create a security violation error