pub struct ProxyProvider { /* private fields */ }Available on non-WebAssembly only.
Proxy authentication provider that delegates to an upstream OAuth server.
This is similar to the TypeScript SDK’s ProxyProvider pattern.
Implementations
Sourceimpl ProxyProvider
impl ProxyProvider
Sourcepub fn new(config: ProxyProviderConfig) -> Self
pub fn new(config: ProxyProviderConfig) -> Self
Create a new proxy provider with the given configuration.
Sourcepub fn with_upstream(upstream_url: impl Into<String>) -> Self
pub fn with_upstream(upstream_url: impl Into<String>) -> Self
Create a proxy provider with just an upstream URL.
Sourcepub fn with_validator_fn<F, Fut>(self, validator: F) -> Self
pub fn with_validator_fn<F, Fut>(self, validator: F) -> Self
Set a custom token validation function. This allows applications to implement custom validation logic.
Sourcepub fn with_validator(self, validator: Arc<dyn TokenValidator>) -> Self
pub fn with_validator(self, validator: Arc<dyn TokenValidator>) -> Self
Set a token validator implementation.
Sourcepub fn introspection_endpoint(self, endpoint: impl Into<String>) -> Self
pub fn introspection_endpoint(self, endpoint: impl Into<String>) -> Self
Set the introspection endpoint.