Available on non-WebAssembly only.
Standard error codes for validation with client elicitation support. Standard error codes for validation with client elicitation support
Provides a consistent set of error codes that clients can use to automate UI elicitation and provide better user experiences.
Structs
- Validation
Error - Structured validation error with elicitation support
Enums
- Validation
Error Code - Standard validation error codes for MCP tools
Traits
- Into
Validation Error - Helper trait for creating validation errors
Functions
- invalid_
format - Create an invalid format error
- missing_
field - Create a missing field error
- not_
allowed - Create a not allowed error
- out_
of_ range - Create an out of range error
- pattern_
mismatch - Create a pattern mismatch error
- security_
violation - Create a security violation error